%Previously Expectant Valence model from Stout & Busemeyer 2002
% Now also includes Prospect Valence Model from Ahn et al., 2008
%By Chris Dancy @ Bucknell University
%For use with IGT-Open software, but only strictly needs some of the
% functions
% This version is optimized for MindModeling (e.g., doesn't have file
%  output functionality
% Input:
%  utilFunStr is a string meant to represent the utility function that
%   should be used by model (e for expectancy or p for prospect)
%  params is a struct that contains all the parameters needed for the given
%   utility function and updating function
% Output:
%  blocks is a 2d array that gives the proportions of deck choices for each 
%   block (1 row per block; 1 block for each 20 trials)

function blocks = mathModel(decks, maxNCards, params, numTrials, numRuns,...
                            utilFunStr, updateFunStr, choiceRuleFunStr)

 %Make sure we have the required params
 for i = (7-nargin):-1:0
     switch i
         %Default for input
         case 0
             %default trial dependent choice rule
             choiceRuleFunStr = 't-d';           
         case 1
             %default delta-learning update function
             updateFunStr = 'delta';             
         case 2
             %defaul expectancy utility function
             utilFunStr = 'e';
         case 3
             numRuns = 100;
         case 4
             outFileNameSubjSim = 'subjSimModelOut';
         case 5
             outFileName = 'simModelOut.csv';
         case 6
             numTrials = 100;
         case 7
             %default parameters for expectancy, delta/decay, & e functions
             params.c = 0.32;
             params.uWeight = 0.35;
             params.alpha = 0.34;
         case 8
             maxNCards = 40;
         case 9
             [maxNCards, decks] = PenaltyDist();
     end
 end
 
 %If we have a nonsensical utilFunStr parameter, set the parameter to the
 % default
 if(~strcmp(utilFunStr, 'e') && ~strcmp(utilFunStr, 'p'))
     utilFunStr = 'e';
 end
 
  %If we have a nonsensical delta learning parameter, set the parameter to
  % the default
 if(~strcmp(updateFunStr, 'delta') && ~strcmp(updateFunStr, 'decay'))
     utilFunStr = 'delta';
 end
 
  %If we have a nonsensical utilFunStr parameter, set the parameter to the
 % default
 if(~strcmp(choiceRuleFunStr, 't-d') && ~strcmp(choiceRuleFunStr, 't-i'))
     utilFunStr = 't-d';
 end

 numDecks = size(decks.index, 2);
 deckExpSave = zeros(numTrials, numDecks);
 totalScoreSave = zeros(numTrials, 1);
 deckPropSave = zeros(numTrials, numDecks);
 
 for k = 1:numRuns
     deckExp = zeros(1, numDecks);
     totalScore = 0;
     totalReward = 0;
     totalPunishment = 0;
     deckProbs = zeros(1, numDecks);
     nCardsSelected = zeros(1, numDecks);
     blocks = zeros(numTrials/20, numDecks);

     for i = 1:numTrials
         dS = 0;
         
         %Save deck expancies for this trial run
         deckExpSave(i,:) = deckExpSave(i,:) + deckExp;
         
         %calculate phi based on number of trials and user supplied (or
         % default) c parameter which (trial-dependent rule)
         if (strcmp(choiceRuleFunStr, 't-d'))
             params.phi = power((i/10), params.c);
         %trial independent choice rule (Ahn et al., 2008)
         elseif( strcmp(choiceRuleFunStr, 't-i') )
             params.phi = power(3, params.c) - 1;
         end
         
         for j = 1:numDecks
             
             %If we've selected the maximum number of cards from jth deck,
             % manually set the probability to a very low value to decrease the
             % chances of selection (or multiple selections)
             if (nCardsSelected(j) == maxNCards)
                deckProbs(j) = 0.0000000001;
             else
                deckProbs(j) = exp(deckExp(j) * params.phi);
             end
         end
         %Normalize probabilities (i.e., define the denominator and use)
         deckProbs = deckProbs / sum(deckProbs);

         %Find the deck selection (dS) based on the multinomial 
         % distribution defined by deck probabilities.
         while( dS == 0 || nCardsSelected(dS) == maxNCards )
             deckNums = mnrnd(1, deckProbs);

             %Select the deck that was selected
             dS = find(deckNums == 1, 1);
         end
         
         nCardsSelected(dS) = nCardsSelected(dS) + 1;
         
         currReward = decks.reward(dS, nCardsSelected(dS));
         totalReward = currReward + totalReward;
         
         currPunish = decks.punish(dS, nCardsSelected(dS));
         totalPunishment = currPunish + totalPunishment;
         
         %Update total score and (selected deck) valence
         totalScore = totalScore + currReward - currPunish;
         totalScoreSave(i) = totalScoreSave(i) + totalScore;
                 
         %calculate the deck utility (either expectancy or propsect (see
         % Ahn et. al., 2008))
         if (strcmp(utilFunStr, 'e'))
            %Deck utility using expectancy function
            deckUtil  = (1 - params.uWeight) * decks.reward(dS, nCardsSelected(dS)) ...
                            - (params.uWeight) * decks.punish(dS, nCardsSelected(dS));
         elseif (strcmp(utilFunStr, 'p'))
            netGain = (decks.reward(dS, nCardsSelected(dS)) - decks.punish(dS, nCardsSelected(dS)));
            %The prospect utility function is piecewise, where the
            % function/equation is dependent upon whether we have a net gain
            % or net loss
            % pAlpha is the prospective utility function alpha (e.g., see
            % Ahn et. al, 2008)
            if (netGain >= 0)
                deckUtil = power(netGain, params.pAlpha);
            else
                deckUtil = params.lambda * power(abs(netGain), params.pAlpha);
            end
         else
             disp('Invalid utility function string, exiting...');
             return
         end
                        
         if (strcmp(updateFunStr, 'delta'))
             %Update expentancy values to make deck selection next trial
             % using delta-learning rule
             deckExp(dS) = deckExp(dS) + ...
                            params.alpha * (deckUtil - deckExp(dS));
         elseif (strcmp(updateFunStr, 'decay'))
             %Update expentancy values to make deck selection next trial
             % using decay-reinforcement rule
             for j=1:numDecks
                 %Always decay the expectancy, but only add the utility if
                 % we've actually selected that deck
                 deckExp(j) = params.alpha * deckExp(j);
                 if (j == ds)
                    deckExp(j) = deckExp(j) + deckUtil;
                 end
             end
         else
             disp('Invalid update (utility learning) function string, exiting...');
             return
         end
         %Update deck count for proportion calculation
         deckPropSave(i,dS) = deckPropSave(i,dS) + 1;
     end
     
 end
 
 %write results to file
 for i = 1:numTrials
     %Used for our output (blocks)
     if (mod(i,20) == 0)
         blockNum = i / 20;
         block = zeros(1,numDecks);
         for j = 1:numDecks
            block(j) = sum(deckPropSave((i-19):i,j)) ...
                       / sum(sum(deckPropSave((i-19):i,:)));
         end
         blocks(blockNum,:) = block;
     end
 end
end

